'use strict';

exports.__esModule = true;
var cssText = 'position:fixed;pointer-events:none;z-index:-9999;opacity:0;';
var copyErrorMessage = 'Failed to copy value to clipboard. Unknown type.';

var $clipboard = function $clipboard(input) {
  var value = void 0;
  if (typeof input !== 'string') {
    try {
      value = JSON.stringify(input);
    } catch (e) {
      throw copyErrorMessage;
    }
  } else {
    value = input;
  }

  var textarea = document.createElement('textarea');

  textarea.value = value;
  textarea.setAttribute('readonly', '');
  textarea.style.cssText = cssText;

  document.body.appendChild(textarea);

  if (navigator.userAgent.match(/ipad|ipod|iphone/i)) {
    textarea.contentEditable = true;
    textarea.readOnly = true;

    var range = document.createRange();

    range.selectNodeContents(textarea);

    var selection = window.getSelection();

    selection.removeAllRanges();
    selection.addRange(range);
    textarea.setSelectionRange(0, 999999);
  } else {
    textarea.select();
  }

  var success = false;

  try {
    success = document.execCommand('copy');
  } catch (err) {
    console.warn(err);
  }

  document.body.removeChild(textarea);

  return success;
};

exports.default = {
  install: function install(Vue) {
    Vue.prototype.$clipboard = $clipboard;
    var generateId = function (id) {
      return function () {
        return '$' + id++;
      };
    }(1);
    var handlers = {};
    var removeHandler = function removeHandler(id) {
      if (id) {
        handlers[id] = null;
        delete handlers[id];
      }
    };
    var addHandler = function addHandler(func) {
      var id = generateId();
      handlers[id] = func;

      return id;
    };
    Vue.directive('clipboard', {
      bind: function bind(el, binding) {
        var arg = binding.arg,
            value = binding.value;


        switch (arg) {
          case 'error':
            var errorHandlerId = addHandler(value);
            el.dataset.clipboardErrorHandler = errorHandlerId;
            return;

          case 'success':
            var successHandlerId = addHandler(value);
            el.dataset.clipboardSuccessHandler = successHandlerId;
            return;

          default:
            var clickEventHandler = function clickEventHandler(event) {
              if (binding.hasOwnProperty('value')) {
                var payload = {
                  value: typeof value === 'function' ? value() : value,
                  event: event
                };

                var handlerId = $clipboard(payload.value) ? el.dataset.clipboardSuccessHandler : el.dataset.clipboardErrorHandler;

                var handler = handlers[handlerId];

                if (handler) {
                  handler(payload);
                }
              }
            };

            var clickEventHandlerId = addHandler(clickEventHandler);

            el.dataset.clipboardClickHandler = clickEventHandlerId;
            el.addEventListener('click', handlers[clickEventHandlerId]);

            return;
        }
      },
      unbind: function unbind(el) {
        var _el$dataset = el.dataset,
            clipboardSuccessHandler = _el$dataset.clipboardSuccessHandler,
            clipboardErrorHandler = _el$dataset.clipboardErrorHandler,
            clipboardClickHandler = _el$dataset.clipboardClickHandler;


        removeHandler(clipboardSuccessHandler);
        removeHandler(clipboardErrorHandler);

        if (clipboardClickHandler) {
          el.removeEventListener('click', handlers[clipboardClickHandler]);
          removeHandler(clipboardClickHandler);
        }
      }
    });
  }
};